package com.fs.his.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.fs.common.utils.DateUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fs.common.utils.StringUtils;
import com.fs.course.domain.FsCourseRedPacketLog;
import com.fs.course.mapper.FsCourseRedPacketLogMapper;
import com.fs.course.vo.FsUserCourseVO;
import com.fs.his.enums.FsUserOperationEnum;
import com.fs.his.param.FsUserOperationLogQueryParam;
import com.fs.his.vo.FsUserOperationLogPageVo;
import com.fs.his.vo.FsUserOperationLogParamVo;
import com.fs.his.vo.FsUserOperationLogVo;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fs.his.mapper.FsUserOperationLogMapper;
import com.fs.his.domain.FsUserOperationLog;
import com.fs.his.service.IFsUserOperationLogService;

/**
 * 用户操作日志Service业务层处理
 *
 * @author fs
 * @date 2025-07-04
 */
@Service
public class FsUserOperationLogServiceImpl extends ServiceImpl<FsUserOperationLogMapper, FsUserOperationLog> implements IFsUserOperationLogService {
    @Autowired
    private FsCourseRedPacketLogMapper fsCourseRedPacketLogMapper;

    /**
     * 查询用户操作日志
     *
     * @param logId 用户操作日志主键
     * @return 用户操作日志
     */
    @Override
    public FsUserOperationLog selectFsUserOperationLogByLogId(Long logId)
    {
        return baseMapper.selectFsUserOperationLogByLogId(logId);
    }

    /**
     * 查询用户操作日志列表
     *
     * @param fsUserOperationLog 用户操作日志
     * @return 用户操作日志
     */
    @Override
    public PageInfo<FsUserOperationLogPageVo> selectFsUserOperationLogList(FsUserOperationLogQueryParam fsUserOperationLog)
    {
        List<FsUserOperationLog> logs = baseMapper.selectFsUserOperationLogList(fsUserOperationLog);
        ArrayList<FsUserOperationLogPageVo> vos = new ArrayList<>();
        if(logs!=null&& !logs.isEmpty()){
            for (FsUserOperationLog log : logs) {
                FsUserOperationLogPageVo vo = new FsUserOperationLogPageVo();
                BeanUtils.copyProperties(log, vo);
                StringBuilder richText = new StringBuilder();
                richText.append("<div>")
                        .append("<div style='font-weight: bold;'>")
                        .append(log.getOperationType())
                        .append("<div>");
                String param = log.getParam();
                if (StringUtils.isNotBlank(param)){
                    FsUserCourseVO courseVO = JSON.parseObject(param, FsUserCourseVO.class);
                    if(courseVO!=null){
                        if (StringUtils.isNotBlank(courseVO.getCourseName())){
                            richText.append("<div style='font-weight: bold;'>").append("课节名称").append("</div>");
                            richText.append("<p>").append(courseVO.getCourseName()).append("<p>");
                        }
                        if (StringUtils.isNotBlank(courseVO.getTrainingCampName())){
                            richText.append("<div style='font-weight: bold;'>").append("所属训练营").append("</div>");
                            richText.append("<p>").append(courseVO.getTrainingCampName()).append("<p>");
                        }
                        if (StringUtils.isNotBlank(courseVO.getTrainingCampName())){
                            richText.append("<div style='font-weight: bold;'>").append("营期名称").append("</div>");
                            richText.append("<p>").append(courseVO.getPeriodName()).append("<p>");
                        }
                    }
                }
                richText.append("</div>");
                vo.setText(richText.toString());
                vos.add(vo);
            }
        }
        PageInfo<FsUserOperationLogPageVo> pageInfo = new PageInfo<>();
        PageInfo<FsUserOperationLog> logPageInfo = new PageInfo<>(logs);
        BeanUtils.copyProperties(logPageInfo, pageInfo);
        pageInfo.setList(vos);
        return pageInfo;
    }

    /**
     * 新增用户操作日志
     *
     * @param fsUserOperationLog 用户操作日志
     * @return 结果
     */
    @Override
    public int insertFsUserOperationLog(FsUserOperationLog fsUserOperationLog)
    {
        fsUserOperationLog.setCreateTime(DateUtils.getNowDate());
        return baseMapper.insertFsUserOperationLog(fsUserOperationLog);
    }

    /**
     * 修改用户操作日志
     *
     * @param fsUserOperationLog 用户操作日志
     * @return 结果
     */
    @Override
    public int updateFsUserOperationLog(FsUserOperationLog fsUserOperationLog)
    {
        return baseMapper.updateFsUserOperationLog(fsUserOperationLog);
    }

    /**
     * 批量删除用户操作日志
     *
     * @param logIds 需要删除的用户操作日志主键
     * @return 结果
     */
    @Override
    public int deleteFsUserOperationLogByLogIds(Long[] logIds)
    {
        return baseMapper.deleteFsUserOperationLogByLogIds(logIds);
    }

    /**
     * 删除用户操作日志信息
     *
     * @param logId 用户操作日志主键
     * @return 结果
     */
    @Override
    public int deleteFsUserOperationLogByLogId(Long logId)
    {
        return baseMapper.deleteFsUserOperationLogByLogId(logId);
    }

    @Override
    public List<String> getOperationType() {
        List<String> labels = Stream.of(FsUserOperationEnum.values())
                .map(FsUserOperationEnum::getLabel)
                .collect(Collectors.toList());
        return labels;
    }

    @Override
    public List<FsUserOperationLogVo> selectFsUserOperationLogByList(FsUserOperationLog fsUserOperationLog) {
        List<FsUserOperationLogVo> list = baseMapper.selectFsUserOperationLogByList(fsUserOperationLog);
        list.forEach(e->{
            FsUserOperationLogParamVo fsUserOperationLogParamVo = JSON.parseObject(e.getParam(), FsUserOperationLogParamVo.class);
            e.setParamVo(fsUserOperationLogParamVo);
            if (ObjectUtil.isEmpty(fsUserOperationLogParamVo)){
                return;
            }
            FsCourseRedPacketLog fsCourseRedPacketLog = fsCourseRedPacketLogMapper.selectFsCourseRedPacketLogByTemporary(fsUserOperationLogParamVo.getVideoId(),e.getUserId());
            e.setFsCourseRedPacketLog(fsCourseRedPacketLog);
        });
        return list;
    }
}
